"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommentModelService = exports.CommentTreeModel = void 0;
const tslib_1 = require("tslib");
/* eslint-disable import/order */
const di_1 = require("@opensumi/di");
const ide_components_1 = require("@opensumi/ide-components");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const index_1 = require("@opensumi/ide-editor/lib/common/index");
const index_2 = require("../../common/index");
const tree_node_defined_1 = require("./tree-node.defined");
const tree_node_module_less_1 = tslib_1.__importDefault(require("./tree-node.module.less"));
let CommentTreeModel = class CommentTreeModel extends ide_components_1.TreeModel {
    constructor(root) {
        super();
        this.init(root);
    }
};
CommentTreeModel = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__param(0, (0, di_1.Optional)()),
    tslib_1.__metadata("design:paramtypes", [tree_node_defined_1.CommentRoot])
], CommentTreeModel);
exports.CommentTreeModel = CommentTreeModel;
let CommentModelService = class CommentModelService extends ide_core_browser_1.Disposable {
    constructor() {
        super();
        // All decoration
        this.selectedDecoration = new ide_components_1.Decoration(tree_node_module_less_1.default.mod_selected); // selected
        this.focusedDecoration = new ide_components_1.Decoration(tree_node_module_less_1.default.mod_focused); // focused
        this._selectedNodes = [];
        this.onDidUpdateTreeModelEmitter = new ide_core_browser_1.Emitter();
        this.disposableCollection = new ide_core_browser_1.DisposableCollection();
        this.applyFocusedDecoration = (target, dispatch = true) => {
            var _a;
            if (target) {
                for (const target of this._selectedNodes) {
                    this.selectedDecoration.removeTarget(target);
                }
                if (this.focusedNode) {
                    this.focusedDecoration.removeTarget(this.focusedNode);
                }
                this.selectedDecoration.addTarget(target);
                this.focusedDecoration.addTarget(target);
                this._focusedNode = target;
                this._selectedNodes = [target];
                dispatch && ((_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange());
            }
        };
        this.removeFocusedDecoration = () => {
            var _a;
            if (this.focusedNode) {
                this.focusedDecoration.removeTarget(this.focusedNode);
                (_a = this.treeModel) === null || _a === void 0 ? void 0 : _a.dispatchChange();
            }
            this._focusedNode = null;
        };
        this.handleTreeBlur = () => {
            this.removeFocusedDecoration();
        };
        this.handleTwistierClick = async (_, node) => {
            if (tree_node_defined_1.CommentFileNode.is(node) || (node === null || node === void 0 ? void 0 : node.isAllowToggle)) {
                this.toggleDirectory(node);
            }
        };
        this.handleItemClick = async (_, node) => {
            this.applyFocusedDecoration(node);
            if (tree_node_defined_1.CommentFileNode.is(node) || (node === null || node === void 0 ? void 0 : node.isAllowToggle)) {
                this.toggleDirectory(node);
            }
            let uri;
            let range;
            if (node instanceof tree_node_defined_1.CommentReplyNode || node instanceof tree_node_defined_1.CommentContentNode) {
                uri = node.thread.uri;
                range = node.thread.range;
            }
            else {
                uri = node.resource;
            }
            this.editorService
                .open(uri, {
                range,
            })
                .then(() => {
                if (node.thread) {
                    node.thread.show();
                }
            });
        };
        this.toggleDirectory = (item) => {
            if (item.expanded) {
                this.commentTreeHandle.collapseNode(item);
            }
            else {
                this.commentTreeHandle.expandNode(item);
            }
        };
        this._whenReady = this.initTreeModel();
    }
    get onDidUpdateTreeModel() {
        return this.onDidUpdateTreeModelEmitter.event;
    }
    get whenReady() {
        return this._whenReady;
    }
    get commentTreeHandle() {
        return this._commentTreeHandle;
    }
    get decorations() {
        return this._decorations;
    }
    get treeModel() {
        return this._treeModel;
    }
    get focusedNode() {
        return this._focusedNode;
    }
    get selectedNodes() {
        return this._selectedNodes;
    }
    async initTreeModel() {
        const childs = await this.commentService.resolveChildren();
        if (!childs) {
            return;
        }
        const root = childs[0];
        if (!root) {
            return;
        }
        this._treeModel = this.injector.get(CommentTreeModel, [root]);
        await this._treeModel.ensureReady;
        this.initDecorations(root);
        this.disposables.push(ide_core_browser_1.Event.any(this.commentService.onThreadsCommentChange, this.commentService.onThreadsChanged, this.commentService.onThreadsCreated)(() => {
            this.refresh();
        }));
        this.onDidUpdateTreeModelEmitter.fire(this._treeModel);
    }
    initDecorations(root) {
        this._decorations = new ide_components_1.DecorationsManager(root);
        this._decorations.addDecoration(this.selectedDecoration);
        this._decorations.addDecoration(this.focusedDecoration);
        return this._decorations;
    }
    handleTreeHandler(handle) {
        this._commentTreeHandle = handle;
    }
    async refresh() {
        await this.whenReady;
        (0, ide_core_browser_1.runWhenIdle)(() => {
            this.treeModel.root.refresh();
        });
    }
    async collapsedAll() {
        await this.whenReady;
        return this.treeModel.root.collapsedAll();
    }
    dispose() {
        this.disposableCollection.dispose();
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(index_2.ICommentsService),
    tslib_1.__metadata("design:type", Object)
], CommentModelService.prototype, "commentService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], CommentModelService.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(index_1.WorkbenchEditorService),
    tslib_1.__metadata("design:type", index_1.WorkbenchEditorService)
], CommentModelService.prototype, "editorService", void 0);
CommentModelService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], CommentModelService);
exports.CommentModelService = CommentModelService;
//# sourceMappingURL=tree-model.service.js.map